//////////////////////////////////////////////////////
// does the sums etc, using mainly algorithms from  //
// numerical recipes 2nd Ed, page 661ff.  the slope //
// and intercept, along with estimates of their     //
// one-parameter uncertainties, and the value of    //
// chi-square (if relevant) and the correlation     //
// coefficient are all calculated.                  //
//////////////////////////////////////////////////////

#include "fitline.h"

void calculate(data* dp, result* rp, flags* fp )
{
    int i ;
    dbl w, sw, sx, sy, sxx, syy, sxy, xbar, ybar, t, stt, styos ;

    w = sw = sx = sy = sxx = syy = sxy = stt = styos = 0.0 ;
    for ( i = 0 ; i < dp->npts ; i++ ) {
        w = dp->s[i] * dp->s[i] ;
        w = 1.0 / w ;                            // weight for ith point
        sw += w ;                                // sum weights
        sx += w * dp->x[i] ;                     // sum w[i]*x[i]
        sy += w * dp->y[i] ;                     // sum w[i]*y[i]
        sxx += w * dp->x[i] * dp->x[i] ;         // sum w[i]*x[i]*x[i]
        syy += w * dp->y[i] * dp->y[i] ;         // sum w[i]*y[i]*y[i]
        sxy += w * dp->x[i] * dp->y[i] ;         // sum w[i]*x[i]*y[i]
    }
    xbar = sx / sw ;
    ybar = sy / sw ;
    for ( i = 0 ; i < dp->npts ; i++ ) {
        t = dp->x[i] - xbar ;
        t /= dp->s[i] ;
        stt += t * t ;
        styos += t * dp->y[i] / dp->s[i] ;
    }
    rp->a = styos / stt ;                        // slope
    rp->b = ( sy - sx * rp->a ) / sw ;           // intercept
    rp->chi2 = syy - rp->a * sxy - rp->b *sy ;   // phimin, actually
    rp->siga = 1.0 / stt ;                       // variance of slope
    rp->sigb = rp->siga * sxx / sw ;             // variance of intercept
    if ( !fp->sflg ) {                           // if no abs "error" estimates
        rp->siga *= rp->chi2 / ( dp->npts -2 ) ; // assume chi2 = n-2
        rp->sigb *= rp->chi2 / ( dp->npts -2 ) ;
    }
    rp->siga = sqrt( rp->siga ) ;                // std dev of slope
    rp->sigb = sqrt( rp->sigb ) ;                // std dev of intercept
    rp->corr = sw * sxy - sx * sy ;
    rp->corr /= sqrt( sw * sxx - sx * sx ) ;
    rp->corr /= sqrt( sw * syy - sy * sy ) ;     // correlation coefficient
    rp->siga *= student( dp->npts -2, .68269 ) ; // multiply by student t
    rp->sigb *= student( dp->npts -2, .68269 ) ;
}
