////////////////////////////////////////////////////////////////////
// uses simpson to calculate the upper limit of the integral      //
// of the "Student" t-distribution function stud(t) such that the //
// integral is equal to conf/2.  This upper limit is the          //
// "Student" t-factor t sub nu for nu degrees of freedom.         //
////////////////////////////////////////////////////////////////////

#include "fitline.h"

dbl student(int nu, dbl conf)
{
    dbl t, sum, h, a, b, y, yold, y0;
    
    a = norm( nu ) / 3.0;    // norm(nu) is the normalization factor
    y0 = conf/2.0;           // y0 is desired area under curve
    h = 0.005;               // h is simpson delta-t
    
    for ( t = 0.0, y = 0.0 ; ; t += 2.0 * h ) {
        sum = stud( t, nu );
        sum += 4.0 * stud( t + h, nu );
        sum += stud( t + 2.0 * h, nu );
        yold = y;
        y += a * h * sum;    // y = integral of stud(t)dt from 0 to 2mh
    
        if ( yold < y0 && y >= y0 ) { // interpolation for last bit
            b = 2.0 * h * ( y0 - yold )/( y - yold );
            return( t + b );
        }
        else if (yold > y0)
            err( "Oh-oh. Couldn't find student t-factor.");
    }
}
